#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/1/23 9:56
# @Author  : chenjw
# @Site    : 
# @File    : html.py
# @Software: PyCharm Community Edition
# @Desc    :  do what
from base64 import b64encode


class Html:
    def __init__(self):
        self.default_css = '''
        .table1{
            line-height: 2em;
            font-family: Arial;
            border-collapse: collapse;
        }
        .thead_tr {
            color: #FF3333;
            background-color: #C0C0C0;
            border-bottom: 1px solid #fff;
        }
        .tbody_tr {
            color: #663333;
            font-size: 0.8em;
            border-bottom: 1px solid #fff;
            background-color: #DDDDDD;
        }
        .tbody_tr:hover {
            background-color: #FFFF33;
        }
        .th1 {
            font-weight: normal;
            text-align: left;
            padding: 0 10px;
            text-align: center;
        }
        '''
        self.default_title = '测试报告'
        self.body_context = ''
        pass

    def setTitle(self, title):
        self.default_title = title
        return self

    def setTable(self, header, detail):
        if isinstance(header, list) is False or isinstance(detail, list) is False:
            return self
        header_msg = ''
        for sin_header in header:
            header_msg += '''<th class="th1">%s</th>\n''' % sin_header
        header_msg = '''<thead><tr class="thead_tr th1">''' + header_msg + '''</tr></thead>'''
        detail_msg = ''
        for sin_line in detail:
            tmp_msg = ''
            for sin_cell in sin_line:
                tmp_msg += '''<td class="th1">%s</td>\n''' % sin_cell
            detail_msg += '''<tr class="tbody_tr th1">''' + tmp_msg + '''</tr>'''
        detail_msg = '''<tbody>''' + detail_msg + '''</tbody>'''
        self.body_context += '''<table class="table1">''' + header_msg + detail_msg + '''</table>'''
        return self

    def setH(self, desc, index=6):
        self.body_context += '''<h%d>%s</h%d>\n''' % (index, desc, index)
        return self

    def draw(x_names=range(10), l_range=[0, 10], r_range=None, x_name=None, l_name=None, r_name='', title=None,
             l_compare=[], r_compare=[], file_name=None, dpi=100):
        import matplotlib.pyplot as plt
        fig = plt.figure()
        ax = fig.add_subplot(111)
        if r_range is not None or r_compare is not None:
            ax2 = ax.twinx()
        l_lns = []
        for sin_l_compare in l_compare:
            data = sin_l_compare['data']
            label = sin_l_compare['label']
            color = sin_l_compare['color']
            lns = ax.plot(x_names, data, color, label=label)
            l_lns.append(lns)

        r_lns = []
        for sin_r_compare in r_compare:
            data = sin_r_compare['data']
            label = sin_r_compare['label']
            color = sin_r_compare['color']
            lns = ax2.plot(x_names, data, color, label=label)
            r_lns.append(lns)

        def add_lns(lns, lns_list):
            for sin_lns in lns_list:
                if lns is None:
                    lns = sin_lns
                else:
                    lns += sin_lns
            return lns

        lns = None
        lns = add_lns(lns, l_lns)
        lns = add_lns(lns, r_lns)
        labs = [l.get_label() for l in lns]
        ax.legend(lns, labs, loc=0)
        ax.grid()

        if x_name is not None:
            ax.set_xlabel(x_name)

        if l_range is not None:
            ax.set_ylim(l_range[0], l_range[1])

        if l_name is not None:
            ax.set_ylabel(l_name)
        if title is not None:
            ax.set_title(title)

        if r_range is not None:
            ax2.tick_params(axis='y', colors='red')  # 刻度颜色
            ax2.spines['right'].set_color('red')  # 纵轴颜色
            ax2.set_ylim(r_range[0], r_range[1])
            if r_name is not None:
                ax2.set_ylabel(r_name)
        if file_name is not None:
            plt.savefig(file_name, dpi=dpi, bbox_inches='tight')

    def setImg(self, x_names=range(10), l_range=[0, 10], r_range=None, x_name=None, l_name=None, r_name='', title=None,
               l_compare=[], r_compare=[], dpi=100):
        file_name = 'default.png'
        self.draw(x_names, l_range, r_range, x_name, l_name, r_name, title, l_compare, r_compare, file_name, dpi)
        with open(file_name, 'rb') as imgFile:
            data = imgFile.read()
            self.body_context += '''<img src="%s"></img>''' % (
            'data:default/png;base64,' + str(b64encode(data), 'utf-8'))


    def save(self, file_name):
        with open(file_name, 'wb') as f:
            msg = '''<!DOCTYPE html>
            <html>
            <head>
            <title>%s</title>
            <style type="text/css">
            %s
            </style>
            </head>
            <body>
            %s
            </body>
            </html>
            ''' % (self.default_title, self.default_css, self.body_context)
            f.write(bytes(msg, 'utf-8'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/1/23 9:56
# @Author  : chenjw
# @Site    : 
# @File    : report.py
# @Software: PyCharm Community Edition
# @Desc    :  do what

from xlsxwriter.workbook import Workbook
import collections


class Report:
    def __init__(self, name):
        self.name = name
        self.workbook = Workbook(name)

    def close(self):
        if self.workbook is not None:
            self.workbook.close()

    # 添加sheet 表
    def add_sheet(self, sheet_name=None):
        if isinstance(sheet_name, str) and sheet_name != '':
            return self.workbook.add_worksheet(sheet_name)
        else:
            return self.workbook.add_worksheet()

    # 设置行的高度
    def setRow(self, sheet, indexs=None, v=None, dict1=None):
        if isinstance(indexs, list) and isinstance(v, int) and v > 0:
            for index in range(indexs):
                if isinstance(index, int) and index > 0:
                    sheet.set_row(index, v)

        if isinstance(dict1, dict) and len(dict1) > 0:
            for _k, _v in dict1.items():
                if isinstance(_k, int) and isinstance(_v, int) and _k >= 0 and _v > 0:
                    sheet.set_row(_k, _v)

    # 默认的格式化东西
    def default_formate(self, num=1, font_size=14, bold=False, color='black'):
        return self.workbook.add_format(
            {'align': 'center',
             'valign': 'vcenter',
             'border': num,
             'font_size': font_size,
             'bold': bold,
             'color': color,
             })

    # 获取 y方向的下标
    def load_sheet_y(self, base, index, size):
        half = int((size + 1) / 2)
        if index >= half:
            return index - half + base
        else:
            return index + base

    # 获取x 方向的下标
    def load_sheet_x(self, base, type, index, size):
        half = int((size + 1) / 2)
        if type == 'key':
            if index >= half:
                return str(chr(ord(base) + 2))
            else:
                return str(chr(ord(base) + 0))
        else:
            if index >= half:
                return str(chr(ord(base) + 3))
            else:
                return str(chr(ord(base) + 1))

    # 拼凑成 一个饼状图需要的数据
    def combine(self, pies, list1, x, y):
        keys = ''
        values = ''
        count = 0
        size = len(list1)
        for index_pie in range(len(pies)):
            pie = pies[index_pie]
            for index_list in range(len(list1)):
                sin = list1[index_list]
                if pie == sin:
                    y_indeed = self.load_sheet_y(y, index_list, size)
                    x_indeed_key = self.load_sheet_x(x, 'key', index_list, size)
                    x_indeed_value = self.load_sheet_x(x, 'value', index_list, size)
                    sin_key = '''$%s$%d''' % (x_indeed_key, y_indeed)
                    sin_value = '''$%s$%d''' % (x_indeed_value, y_indeed)
                    if index_pie == len(pies) - 1:
                        keys += sin_key
                        values += sin_value
                    else:
                        keys += sin_key + ':'
                        values += sin_value + ':'
                    count += 1
                    break
        return keys, values, count

    # 用于填充颜色
    def fillcolor(self, count):
        colors = ['orange', 'blue', 'green', 'cyan', 'yellow', 'pink', 'red', 'gray', 'lime', 'brown', 'magenta',
                  'navy', 'purple', 'silver', 'white', 'black']
        res = []
        for i in range(count):
            i = i % len(colors)
            res.append({'fill': {'color': colors[i]}})
        return res

    # 测试概况
    def summary(self, sheet_name, title, dict1, pies=None, pie_name='default', x_offset=25, y_offset=10):
        if isinstance(dict1, collections.OrderedDict) is False:
            return
        list1 = []
        for k, v in dict1.items():
            list1.append(k)
        summary_sheet = self.add_sheet(sheet_name)
        # 自定义 cell 的大小
        summary_sheet.set_column("A:D", 20)
        row_size = int((len(list1) + 1) / 2)
        self.setRow(summary_sheet, [lambda x: x + 1, [x for x in range(row_size)]])
        # 自定义样式
        title_formate = self.default_formate(font_size=18, bold=True, color='blue')
        common_formate = self.default_formate()
        # 标题
        summary_sheet.merge_range('A1:D1', title, title_formate)
        # 详情
        x = 'A'
        y = 2
        size = len(list1)
        for index in range(len(list1)):
            y_indeed = self.load_sheet_y(y, index, size)
            x_indeed_key = self.load_sheet_x(x, 'key', index, size)
            x_indeed_value = self.load_sheet_x(x, 'value', index, size)
            summary_sheet.write(x_indeed_key + str(y_indeed), list1[index], common_formate)
            summary_sheet.write(x_indeed_value + str(y_indeed), dict1[list1[index]], common_formate)

        current_index = (len(list1) + 1) / 2 + 1

        # 此处为饼状图的分隔符
        if pies is not None:
            keys, values, count = self.combine(pies, list1, x, y)
            if count == 0:
                return
            chart = self.workbook.add_chart({'type': 'pie'})

            serie = {
                'name': pie_name,
                'categories': '''=%s!%s''' % (sheet_name, keys),
                'values': '''=%s!%s''' % (sheet_name, values),
                'points': self.fillcolor(count),
            }
            chart.add_series(serie)
            summary_sheet.insert_chart('A' + str(current_index + 2), chart,
                                       {'x_offset': x_offset, 'y_offset': y_offset})

    def detail(self, sheet_name, dict1, list1, width=20):
        if isinstance(dict1, collections.OrderedDict) is False:
            return
        if isinstance(list1, list) is False:
            return

        detail_sheet = self.add_sheet(sheet_name)
        dst_A = chr(int(ord('A') + len(dict1) - 1))

        # 自定义 cell 的大小
        detail_sheet.set_column('''A:%s''' % (dst_A), width)
        self.setRow(detail_sheet, [lambda x: x + 1, [x for x in range(len(list1) + 2)]])

        # 初始化样式
        table_header_formate = self.default_formate(font_size=16, bold=True)
        common_formate = self.default_formate()

        # 设置表头
        index_x = 0
        for key, value in dict1.items():
            detail_sheet.write('''%s1''' % (chr(index_x + ord('A'))), value, table_header_formate)
            index_x += 1
        # 设置内容
        index_y = 2
        for sin_list in list1:
            if isinstance(sin_list, dict):
                index_x = 0
                for key, _tmp in dict1.items():
                    if key in sin_list.keys():
                        value = sin_list[key]
                    else:
                        value = '-'
                    detail_sheet.write('''%s%d''' % (chr(index_x + ord('A')), index_y), value, common_formate)
                    index_x += 1
                index_y += 1
        # 自动过滤
        detail_sheet.autofilter('''A1:%s%d''' % (dst_A, len(list1) + 1))

if __name__ == '__main__':
    pass